@@ -52,363 +52,6 @@ thread::thread() : DetachedThread(stack::sip.stacksize)
 	session = NULL;
 }
 
-bool thread::assign(stack::call *cr, unsigned count)
-{
-	if(cr->rtp)
-		return true;
-
-	cr->rtp = rtpproxy::create(count);
-	if(cr->rtp)
-		return true;
-
-	return false;
-}
-
-void thread::inviteRemote(stack::session *s, const char *uri_target)
-{
-	assert(s != NULL && s->parent != NULL);
-	assert(uri_target != NULL);
-
-	Socket::address resolve;
-	stack::session *invited;
-	stack::call *call = s->parent;
-	linked_pointer<stack::segment> sp = call->segments.begin();
-	char username[MAX_USERID_SIZE];
-	char touri[MAX_URI_SIZE];
-	char route[MAX_URI_SIZE];
-	osip_message_t *invite = NULL;
-	char expheader[32];
-	char seqid[64];
-	int cid;
-	unsigned count = 0;
-
-	// make sure we do not re-invite an existing active member again
-	while(is(sp)) {
-		if(!stricmp(sp->sid.identity, uri_target) && sp->sid.state == stack::session::OPEN)
-			return;
-		sp.next();
-	}
-	
-	snprintf(touri, sizeof(touri), "<%s>", uri_target);
-
-	proxyinfo.clear();
-	server::classify(&proxyinfo, &call->source->proxy, NULL);
-	invite = NULL;
-
-	eXosip_lock();
-	if(eXosip_call_build_initial_invite(&invite, touri, s->from, NULL, call->subject)) {
-		process::errlog(ERRLOG, "cannot invite %s; build failed", uri_target);
-		eXosip_unlock();
-		return;
-	}
-
-	if(destination == FORWARDED)
-		switch(call->forwarding) {
-		case stack::call::FWD_ALL:
-			stack::sipPublish(&iface, route, call->refer, sizeof(route));
-			snprintf(touri, sizeof(touri), "<%s>;reason=unconditional", route);
-			osip_message_set_header(invite, "Diversion", touri);
-			break;
-		case stack::call::FWD_NA:
-			stack::sipPublish(&iface, route, call->refer, sizeof(route));
-			snprintf(touri, sizeof(touri), "<%s>;reason=no-answer", route);
-			osip_message_set_header(invite, "Diversion", touri);
-			break;
-		case stack::call::FWD_BUSY:
-			stack::sipPublish(&iface, route, call->refer, sizeof(route));
-			snprintf(touri, sizeof(touri), "<%s>;reason=user-busy", route);
-			osip_message_set_header(invite, "Diversion", touri);
-			break;
-		case stack::call::FWD_DND:
-			stack::sipPublish(&iface, route, call->refer, sizeof(route));
-			snprintf(touri, sizeof(touri), "<%s>;reason=do-not-disturb", route);
-			osip_message_set_header(invite, "Diversion", touri);
-			break;
-		case stack::call::FWD_AWAY:
-			stack::sipPublish(&iface, route, call->refer, sizeof(route));
-			snprintf(touri, sizeof(touri), "<%s>;reason=away", route);
-			osip_message_set_header(invite, "Diversion", touri);
-			break;
-		default:
-			break;
-		}
-
-	osip_message_set_header(invite, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-	osip_message_set_header(invite, ALLOW_EVENTS, "talk, hold, refer");
-	osip_message_set_supported(invite, "100rel,replaces,timer");
-
-	if(call->expires) {
-		snprintf(expheader, sizeof(expheader), "%ld", call->expires);
-		osip_message_set_header(invite, SESSION_EXPIRES, expheader);
-	}
-
-	osip_message_set_body(invite, s->sdp, strlen(s->sdp));
-	osip_message_set_content_type(invite, "application/sdp");
-	stack::siplog(invite);
-	cid = eXosip_call_send_initial_invite(invite);
-	if(cid > 0) {
-		snprintf(seqid, sizeof(seqid), "%08x-%d", s->sequence, s->cid);
-		stack::sipAddress(&iface, route, seqid, sizeof(route));
-		eXosip_call_set_reference(cid, route);
-		++count;
-	}
-	else {
-		process::errlog(ERRLOG, "invite failed for %s", uri_target);
-		eXosip_unlock();
-		return;
-	}
-		
-	eXosip_unlock();
-	invited = stack::create(call, cid);
-	registry::incUse(NULL, stats::OUTGOING);
-	rtpproxy::copy(&invited->proxy, &proxyinfo);
-	uri::userid(uri_target, username, sizeof(username));
-	uri::hostid(uri_target, route, sizeof(route));
-	String::set(invited->identity, sizeof(invited->identity), uri_target);
-	String::set(invited->display, sizeof(invited->display), username);
-	snprintf(invited->from, sizeof(invited->from), "<%s>", uri_target);
-	resolve.set(route, 5060);
-	if(resolve.getAddr())
-		uri::identity(resolve.getAddr(), invited->sysident, username, sizeof(invited->sysident));
-	else
-		snprintf(invited->sysident, sizeof(invited->sysident), "%s@unknown", username);
-
-	switch(destination) {
-	case FORWARDED:
-		debug(3, "forwarding to %s\n", uri_target);
-		break;
-	default:
-		debug(3, "inviting %s\n", uri_target);
-	}
-}
-
-void thread::inviteLocal(stack::session *s, registry::mapped *rr)
-{
-	assert(s != NULL && s->parent != NULL);
-	assert(rr != NULL);
-
-	linked_pointer<registry::target> tp = rr->internal.targets;
-	stack::session *invited;
-	stack::call *call = s->parent;
-	linked_pointer<stack::segment> sp = call->segments.begin();
-
-	time_t now;
-	osip_message_t *invite;
-	char expheader[32];
-	char seqid[64];
-	char route[MAX_URI_SIZE];
-	char touri[MAX_URI_SIZE];
-	int cid;
-	unsigned count = 0;
-
-	time(&now);
-
-	if(rr->expires && rr->expires < now + 1)
-		return;
-
-	// make sure we do not re-invite an existing active member again
-	while(is(sp)) {
-		if(sp->sid.reg == rr && sp->sid.state == stack::session::OPEN) {
-			return;
-		}
-		sp.next();
-	}
-
-	switch(rr->status) {
-	case MappedRegistry::IDLE:
-		break;
-	case MappedRegistry::BUSY:
-		if(!call->count && call->forwarding == stack::call::FWD_NA)
-			call->forwarding = stack::call::FWD_BUSY;
-		return;
-	case MappedRegistry::DND:
-		if(!call->count && call->forwarding == stack::call::FWD_NA)
-			call->forwarding = stack::call::FWD_DND;
-		return;
-	case MappedRegistry::AWAY:
-		if(!call->count && call->forwarding == stack::call::FWD_NA)
-			call->forwarding = stack::call::FWD_AWAY;
-		return;
-	default:
-		return;
-	}
-	
-	while(is(tp)) {
-		invited = NULL;
-		if(tp->expires && tp->expires < now + 1)
-			goto next;
-
-		switch(tp->status) {
-		case registry::target::READY:
-			break;
-
-		case registry::target::AWAY:
-			if(call->count)
-				goto next;
-			if(call->forwarding == stack::call::FWD_NA)
-				call->forwarding = stack::call::FWD_AWAY;
-			goto next;
-		case registry::target::DND:
-			if(call->count)
-				goto next;
-			if(call->forwarding == stack::call::FWD_NA || call->forwarding == stack::call::FWD_AWAY)
-				call->forwarding = stack::call::FWD_DND;
-			goto next;
-		case registry::target::BUSY:
-			if(call->count)
-				goto next;
-			if(call->forwarding != stack::call::FWD_IGNORE)
-				call->forwarding = stack::call::FWD_BUSY;
-			goto next;
-		default:
-			goto next;
-		}
-
-		proxyinfo.clear();
-
-		// if proxy required, but not available, then we must skip this
-		// invite...
-		if(server::classify(&proxyinfo, &call->source->proxy, (struct sockaddr *)&tp->address) && !assign(call, 4))
-			goto next;
-
-		invite = NULL;
-		eXosip_lock();
-
-		if(destination == ROUTED) {
-			stack::sipPublish(&tp->address, route, call->dialed, sizeof(route));
-			snprintf(touri, sizeof(touri), "\"%s\" <%s;user=phone>", call->dialed, route);
-		}
-		else if(call->phone)
-			snprintf(touri, sizeof(touri), "<%s;user=phone>", tp->contact);
-		else
-			snprintf(touri, sizeof(touri), "<%s>", tp->contact);
-
-		stack::sipPublish(&tp->address, route + 1, NULL, sizeof(route) - 5);
-		route[0] = '<';
-		String::add(route, sizeof(route), ";lr>");
-		if(eXosip_call_build_initial_invite(&invite, touri, s->from, route, call->subject)) {
-			stack::sipPublish(&tp->address, route, NULL, sizeof(route));
-			process::errlog(ERRLOG, "cannot invite %s; build failed", route);
-			goto unlock;
-		}
-
-		// if not routing, then separate to from request-uri for forwarding
-		if(destination != ROUTED) {
-			stack::sipPublish(&tp->address, route, call->dialed, sizeof(route));
-			if(call->phone)
-				String::add(route, sizeof(route), ";user=phone");
-			snprintf(touri, sizeof(touri), "\"%s\" <%s>", call->dialed, route);
-			if(invite->to) {
-				osip_to_free(invite->to);
-				invite->to = NULL;
-			}
-			osip_message_set_to(invite, touri);
-		}
-
-		if(destination == FORWARDED) {
-			switch(call->forwarding) {
-			case stack::call::FWD_ALL:
-				stack::sipPublish(&tp->iface, route, call->refer, sizeof(route));
-				snprintf(touri, sizeof(touri), "<%s>;reason=unconditional", route);
-				osip_message_set_header(invite, "Diversion", touri);
-				break;
-			case stack::call::FWD_NA:
-                stack::sipPublish(&tp->iface, route, call->refer, sizeof(route));
-                snprintf(touri, sizeof(touri), "<%s>;reason=no-answer", route);
-                osip_message_set_header(invite, "Diversion", touri);
-                break;
-			case stack::call::FWD_BUSY:
-                stack::sipPublish(&tp->iface, route, call->refer, sizeof(route));
-                snprintf(touri, sizeof(touri), "<%s>;reason=user-busy", route);
-                osip_message_set_header(invite, "Diversion", touri);
-                break;
-			case stack::call::FWD_DND:
-                stack::sipPublish(&tp->iface, route, call->refer, sizeof(route));
-                snprintf(touri, sizeof(touri), "<%s>;reason=do-not-disturb", route);
-                osip_message_set_header(invite, "Diversion", touri);
-                break;
-			case stack::call::FWD_AWAY:
-                stack::sipPublish(&tp->iface, route, call->refer, sizeof(route));
-                snprintf(touri, sizeof(touri), "<%s>;reason=away", route);
-                osip_message_set_header(invite, "Diversion", touri);
-                break;
-			default:
-				break;
-			}
-		}
-
-		osip_message_set_header(invite, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
-		osip_message_set_header(invite, ALLOW_EVENTS, "talk, hold, refer");
-		osip_message_set_supported(invite, "100rel,replaces,timer");
-
-		if(call->expires) {
-			snprintf(expheader, sizeof(expheader), "%ld", call->expires);
-			osip_message_set_header(invite, SESSION_EXPIRES, expheader);
-		}
-
-		osip_message_set_body(invite, s->sdp, strlen(s->sdp));
-		osip_message_set_content_type(invite, "application/sdp");
-		stack::siplog(invite);
-		cid = eXosip_call_send_initial_invite(invite);
-		if(cid > 0) {
-			snprintf(seqid, sizeof(seqid), "%08x-%d", s->sequence, s->cid);
-			stack::sipAddress(&tp->iface, route, seqid, sizeof(route));
-			eXosip_call_set_reference(cid, route);
-			++count;
-		}
-		else {
-			stack::sipPublish(&tp->address, route, NULL, sizeof(route));
-			process::errlog(ERRLOG, "invite failed for %s", route);
-			goto unlock;
-		}
-		
-		eXosip_unlock();
-
-		invited = stack::create(call, cid);
-		rtpproxy::copy(&invited->proxy, &proxyinfo);
-		
-		if(rr->ext) 
-			snprintf(invited->sysident, sizeof(invited->sysident), "%u", rr->ext);
-		else
-			String::set(invited->sysident, sizeof(invited->sysident), rr->userid);
-		if(rr->display[0])
-			String::set(invited->display, sizeof(invited->display), rr->display);
-		else
-			String::set(invited->display, sizeof(invited->display), invited->sysident);
-		stack::sipPublish(&tp->iface, invited->identity, invited->sysident, sizeof(invited->identity));
-		if(rr->ext && !rr->display[0])
-			snprintf(invited->from, sizeof(invited->from), 
-				"\"%s\" <%s;user=phone>", invited->sysident, invited->identity);
-		else if(rr->display[0])
-			snprintf(invited->from, sizeof(invited->from),
-				"\"%s\" <%s>", rr->display, invited->identity);
-		else
-			snprintf(invited->from, sizeof(invited->from),
-				"<%s>", invited->identity);
-		registry::incUse(rr, stats::OUTGOING);
-		invited->reg = rr;
-
-		stack::sipPublish(&tp->address, route, NULL, sizeof(route));
-		switch(destination) {
-		case FORWARDED:
-			debug(3, "forwarding to %s\n", route);
-			break;
-		case ROUTED:
-			debug(3, "routing to %s\n", route);
-			break;
-		default:
-			debug(3, "inviting %s\n", route);
-		}
-		goto next;	 
-unlock:
-		eXosip_unlock();
-next:
-		tp.next();
-	}
-	if(count && destination == FORWARDED)
-		String::set(call->refer, sizeof(call->refer), rr->userid);		
-}
-
 void thread::message(void)
 {
 	osip_content_type_t *ct;
@@ -484,19 +127,21 @@ void thread::invite(void)
 	char fromext[32];
 	cdr *cdrnode;
 
+	memcpy(&session->parent->iface, &iface, sizeof(iface));
+
 	// FIXME: we should get proxy count extimate from sdp into global thread object...
 
 	// assign initial proxy if required to accept call...
 	// if no proxy available, then return 503...
 	if(server::classify(&session->proxy, &call->source->proxy, via_address.getAddr())) {
-		if(!assign(call, 4)) {
+		if(!stack::assign(call, 4)) {
 noproxy:
 			send_reply(SIP_SERVICE_UNAVAILABLE);
 			call->failed(this, session);
 			return;
 		}
 	}
-	else if(destination == EXTERNAL && server::isProxied() && !assign(call, 4))
+	else if(destination == EXTERNAL && server::isProxied() && !stack::assign(call, 4))
 		goto noproxy;
 		
 	if(extension)
@@ -536,8 +181,6 @@ noproxy:
 	else if(reginfo) {
 		target = reginfo->userid;
 		toext = reginfo->ext;
-		call->fwdmask = server::getForwarding(reginfo->userid);
-		String::set(call->refer, sizeof(call->refer), reginfo->userid);
 	}
 
 	time(&call->starting);
@@ -657,7 +300,7 @@ noproxy:
 		String::set(cdrnode->display, sizeof(cdrnode->display), session->display);
 		cdr::post(cdrnode);
 
-		inviteRemote(session, requesting);
+		stack::inviteRemote(session, requesting);
 		session->closed = false;
 		goto exit;
 	case ROUTED:
@@ -712,8 +355,8 @@ noproxy:
 		server::release(dialed);
 
 		String::set(session->parent->forward, MAX_USERID_SIZE, reginfo->userid);
-		session->parent->forwarding = stack::call::FWD_NA;
-		inviteLocal(session, reginfo);
+		session->parent->forwarding = "na";
+		stack::inviteLocal(session, reginfo, destination);
 	}
 
 	if(dialed.keys) {
@@ -721,7 +364,7 @@ noproxy:
 	}
 
 exit:
-	if(!call->invited) {
+	if(!call->invited && !stack::forward(session->parent)) {
 		call->busy(this);
 		return;
 	}
@@ -1856,6 +1499,7 @@ void thread::run(void)
 		switch(sevent->type) {
 		case EXOSIP_REGISTRATION_FAILURE:
 			stack::siplog(sevent->response);
+			debug(4, "sip: registration response %d", sevent->response->status_code); 
 			if(sevent->response && sevent->response->status_code == 401) {
 				sip_realm = NULL;
 				proxy_auth = (osip_proxy_authenticate_t*)osip_list_get(OSIP2_LIST_PTR sevent->response->proxy_authenticates, 0);
@@ -1953,6 +1597,7 @@ void thread::run(void)
 			session = stack::access(sevent->cid);
 			if(!session)
 				break;
+			debug(4, "sip: call response %d\n", sevent->response->status_code);
 			switch(sevent->response->status_code) {
 			case SIP_DECLINE:
 			case SIP_MOVED_PERMANENTLY:
